import { wrapperLoggedIn } from "lib/route_wrapper/wrapperLoggedIn";
import { NextRequest, NextResponse } from "next/server";

import { checkVerified } from "./action";

export const GET = wrapperLoggedIn(GetVerify);

async function GetVerify(req: NextRequest): Promise<NextResponse<unknown>> {
    const { searchParams } = new URL(req.url);
    const { user, error } = await checkVerified(searchParams.get("token"));

    if (error) {
        return NextResponse.json(
            {
                error,
            },
            {
                status: 400,
            },
        );
    }
    if (!user.emailValidated) {
        return NextResponse.redirect(new URL("/verify/email", req.url));
    }

    return NextResponse.redirect(new URL("/frontpage", req.url));
}
