"use client";
import { createContext, useContext, useEffect, useState } from "react";

import { useSocketContext } from "../useSocketContext";
import { type EleMap } from "../useViewerContext.tsx_o";

import { OverlayWorker } from "components/editor/worker/overlayWorker";
import { PageWorker } from "components/editor/worker/pageWorker";

interface WorkerContext {
    worker: EleMap<PageWorker>;
    overlayWorker: EleMap<OverlayWorker>;
}

const WorkerContext = createContext<WorkerContext>({
    worker: new Map(),
    overlayWorker: new Map(),
});

export function WorkerContextProvider({
    n,
    children,
}: {
    n: number;
    children: React.ReactNode;
}) {
    const { socket } = useSocketContext();

    const [worker, setWorker] = useState<EleMap<PageWorker>>(new Map());
    const [overlayWorker, setOverlayWorker] = useState<EleMap<OverlayWorker>>(
        new Map(),
    );

    /* Scaling and create the workers dynamically */
    useEffect(() => {
        let w = worker;
        let o = overlayWorker;

        //Scale up
        if (w.size < n) {
            for (let i = w.size; i < n; i++) {
                w.set(w.size, new PageWorker());
                o.set(o.size, new OverlayWorker(socket));
            }
            w = new Map(w);
            o = new Map(o);
            setWorker(w);
            setOverlayWorker(o);
        }

        //Scale down
        if (w.size > n) {
            for (let i = w.size - 1; i >= n; i--) {
                w.get(i)
                    .terminate()
                    .then(() => {
                        w.delete(i);
                    });
                o.get(i).terminate();
                o.delete(i);
            }

            w = new Map(w);
            o = new Map(o);
            setWorker(w);
            setOverlayWorker(o);
        }
        console.log("WorkerContextProvider", w.size, o.size);
    }, [n, overlayWorker, socket, worker]);

    return (
        <WorkerContext.Provider value={{ worker, overlayWorker }}>
            {children}
        </WorkerContext.Provider>
    );
}

export function useWorkerContext() {
    const context = useContext(WorkerContext);
    if (!context) {
        throw new Error(
            "useWorkerContext must be used within a WorkerContextProvider",
        );
    }
    return context;
}

export function useWorker(idx: number): {
    worker: PageWorker | undefined;
    overlayWorker: OverlayWorker | undefined;
} {
    const { worker, overlayWorker } = useWorkerContext();
    return {
        worker: worker?.get(idx),
        overlayWorker: overlayWorker?.get(idx),
    };
}
