import { useCallback, useEffect } from "react";
import { Button, ButtonGroup } from "react-bootstrap";
import {
    TbArrowsDownUp,
    TbArrowsLeftRight,
    TbCut,
    TbFlipHorizontal,
    TbFlipVertical,
    TbRotate,
    TbRotateClockwise,
} from "react-icons/tb";

import { ImageSnip } from "@snip/snips/general/image";

import DragMove from "../../common/drag";
import { Help, HelpDescription, HelpHeader } from "../../common/help";
import {
    Overlay,
    OverlayContent,
    OverlayHandles,
    OverlayOutline,
    Position,
} from "../../common/overlay";
import { ItemSmall, Submenu } from "../../common/submenu";
import { Toggle } from "../../common/toggle";
import { ToolComponents, ToolSubmenuProps, ToolToggleProps } from "../../types";
import { useImageEditorContext } from "../context";
import { ImageOverlay } from "../overlay";
import { CutHandlersOverlayOutline } from "./cutHandler";

import { useQueuedSnipsContext } from "components/context/socket/useQueuedSnipsContext";
import { useWorker } from "components/context/viewer/useWorkerContext";
import { useEditorContext } from "components/editor/context";
import { Input } from "components/form/inputs/base";

import styles from "./cut.module.scss";

function CutToggle(props: ToolToggleProps) {
    const { activeSnipId } = useEditorContext();
    const { snips } = useQueuedSnipsContext();

    // this tool only works with an active image snip
    if (!activeSnipId || !(snips.get(activeSnipId) instanceof ImageSnip)) {
        return null;
    }

    return <Toggle label="Resize" icon={<TbCut />} {...props} />;
}

function CutHelp() {
    return (
        <Help>
            <HelpHeader>
                <span>Cut</span>
                <TbCut />
            </HelpHeader>
            <HelpDescription>
                <p>Using this tool you may cut the image.</p>
            </HelpDescription>
        </Help>
    );
}

function RotateButtons(props) {
    return (
        <Input label="Rotate" type="none" as="div" {...props}>
            <ButtonGroup>
                <Button variant="none">
                    <TbRotate />
                </Button>
                <Button variant="none">
                    <TbRotateClockwise />
                </Button>
            </ButtonGroup>
        </Input>
    );
}

function MirrorButtons(props) {
    return (
        <Input label="Mirror" type="none" as="div" {...props}>
            <ButtonGroup>
                <Button variant="none">
                    <TbFlipVertical />
                </Button>
                <Button variant="none">
                    <TbFlipHorizontal />
                </Button>
            </ButtonGroup>
        </Input>
    );
}
