// eslint-disable-next-line @typescript-eslint/no-var-requires
const { PHASE_DEVELOPMENT_SERVER } = require('next/constants');
const withMDX = require('@next/mdx')();
const withBundleAnalyzer = require("@next/bundle-analyzer")({
    enabled: process.env.ANALYZE === "true",
});


const filter_client = (resource, context) => {
    switch (resource) {
        case "sharp":
        case "canvas":
        case "@snip/utils/config":
            console.log("Ignoring module (client-side): " + resource);
            return true;
        default:
            return false;
    }
};


/** @type {import('next').NextConfig} */
module.exports = (phase,) => {
    const CONFIG_SHARED = {
        distDir: "build",
        poweredByHeader: false,
        eslint: {
            dirs: [
                "lib",
                "components",
                "app",
                "controller",
            ],
        },
        reactStrictMode: true,
        swcMinify: true,
        pageExtensions: ['js', 'jsx', 'mdx', 'ts', 'tsx'],
        webpack: (config, { isServer, webpack }) => {
            // Fixes npm packages that depend on `fs` module
            if (!isServer) {
                // fs and path
                config.plugins.push(new webpack.IgnorePlugin({ checkResource: filter_client }));
                config.plugins.push(
                    // Remove node: from import specifiers, because Next.js does not yet support node: scheme
                    // https://github.com/vercel/next.js/issues/28774
                    new webpack.NormalModuleReplacementPlugin(
                        /^node:/,
                        (resource) => {
                            resource.request = resource.request.replace(/^node:/, '');
                        },
                    ),
                );
                config.plugins.push(
                    new webpack.IgnorePlugin({
                        resourceRegExp: /\/__test__\//,
                    })
                );
                config.resolve.fallback = {
                    fs: false,
                    path: false,
                }
            }
            config.externals.push({ canvas: 'commonjs canvas' });
            config.externals.push({ "@snip/render": 'commonjs @snip/render' });
            config.externals.push({ "@snip/database": 'commonjs @snip/database' });
            return config;
        },
    }


    if (phase === PHASE_DEVELOPMENT_SERVER) {
        return withBundleAnalyzer(withMDX({
            /* development only config options here */
            ...CONFIG_SHARED,
            compress: false,
        }))
    }

    return withMDX({
        /* config options for all phases except development here */
        ...CONFIG_SHARED,
        compress: true,

        compiler: {
            removeConsole: {
                exclude: ['error'],
            },
        }
    })
}

